#pragma once

#include <stdint.h>
#include <avr/eeprom.h>
#include "devices/mcu/atmega328.hpp"
#include "devices/hd44780.hpp"
#include "devices/character_display.hpp"
#include "devices/ttl74595.hpp"
#include "range.hpp"


namespace Voltmeter
{
    class Board
    {
        public:

            using Mcu = Devices::Mcu::Atmega328;
            using WatchDog = Devices::Mcu::WatchDog::AlwaysOn;

            using DisplayRegister = 
                Devices::Ttl74595<
                    Mcu::PortD::Pin7, // Serial Out
                    Mcu::PortB::Pin1, // Serial Clk
                    Mcu::PortB::Pin0, // Latch Clk
                    1 // N
                >;
            
            using Display = 
                Devices::CharacterDisplay<
                    Devices::Hd44780<
                        Devices::Hd44780TransportRegister<
                            DisplayRegister,
                            1, // RS
                            0, // RW
                            2, // EN
                            3, // DB4
                            4, // DB5
                            5, // DB6
                            6, // DB7
                            7  // Backlight
                        >
                    >
                >;

            using PowerEnablePin = Mcu::PortC::Pin0;

            using OnOffButtonPin = Mcu::PortC::Pin1;
            using RangeButtonPin = Mcu::PortC::Pin2;
            using TalkButtonPin = Mcu::PortC::Pin3;

            using SelectAPin = Mcu::PortC::Pin5;
            using SelectBPin = Mcu::PortC::Pin4;

            using DacMosiPin = Mcu::PortD::Pin1;
            using DacCsPin = Mcu::PortD::Pin2;
            using DacSckPin = Mcu::PortD::Pin4;

            using DacSpi = Board::Mcu::UsartSpi;

            using SdCsPin = Mcu::PortB::Pin2;
            using SdMosiPin = Mcu::PortB::Pin3;
            using SdMisoPin = Mcu::PortB::Pin4;
            using SdSckPin = Mcu::PortB::Pin5;

            using Adc = Mcu::Adc;
            using AdcTimer = Mcu::Timer0;

            static constexpr uint16_t SamplingFrequency = 10000;

            static void PreInit();
            static void Init();
            static void SetRange(Range const range);

            static void SetTimeout(WatchDog::Timeout timeout);
            static void Nop();
            static void Loop();

            template <class Pin>
            static void WaitForPin(bool negated)
            {
                while (Pin::Get() ^ negated)
                {
                    Nop();
                }
            }
    };
}